Package: BranchGLM
Type: Package
Title: Efficient Best Subset Selection for GLMs via Branch and Bound
        Algorithms
Version: 3.0.1
Date: 2024-9-27
Authors@R: person("Jacob", "Seedorff", , "jacob-seedorff@uiowa.edu", role = c("aut", "cre"))
Maintainer: Jacob Seedorff <jacob-seedorff@uiowa.edu>
URL: https://github.com/JacobSeedorff21/BranchGLM
BugReports: https://github.com/JacobSeedorff21/BranchGLM/issues
Description: Performs efficient and scalable glm best 
	subset selection using a novel implementation of a branch and bound algorithm.
	To speed up the model fitting process, a range of optimization
	methods are implemented in 'RcppArmadillo'. Parallel computation 
	is available using 'OpenMP'.
License: Apache License (>= 2)
Depends: R (>= 3.3.0)
Imports: Rcpp (>= 1.0.7), methods, stats, graphics
LinkingTo: Rcpp, RcppArmadillo, BH
RoxygenNote: 7.3.2
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2024-09-28 03:37:04 UTC; Jacob
Author: Jacob Seedorff [aut, cre]
Repository: CRAN
Date/Publication: 2024-09-28 04:00:02 UTC
Built: R 4.4.1; x86_64-apple-darwin20.6.0; 2024-09-28 13:18:05 UTC; unix
