# Generated by roxygen2: do not edit by hand

S3method(Cindex,BranchGLM)
S3method(Cindex,BranchGLMROC)
S3method(Cindex,numeric)
S3method(ROC,BranchGLM)
S3method(ROC,numeric)
S3method(Table,BranchGLM)
S3method(Table,numeric)
S3method(VariableImportance.boot,BranchGLMVI)
S3method(VariableImportance.boot,BranchGLMVS)
S3method(VariableSelection,BranchGLM)
S3method(VariableSelection,formula)
S3method(barplot,BranchGLMVI)
S3method(boxplot,BranchGLMVI.boot)
S3method(coef,BranchGLM)
S3method(coef,BranchGLMVS)
S3method(coef,summary.BranchGLMVS)
S3method(confint,BranchGLM)
S3method(deviance,BranchGLM)
S3method(extractAIC,BranchGLM)
S3method(family,BranchGLM)
S3method(formula,BranchGLM)
S3method(hist,BranchGLMVI.boot)
S3method(logLik,BranchGLM)
S3method(model.frame,BranchGLM)
S3method(nobs,BranchGLM)
S3method(plot,BranchGLM)
S3method(plot,BranchGLMCIs)
S3method(plot,BranchGLMROC)
S3method(plot,BranchGLMVS)
S3method(plot,summary.BranchGLMVS)
S3method(predict,BranchGLM)
S3method(predict,BranchGLMVS)
S3method(predict,summary.BranchGLMVS)
S3method(print,BranchGLM)
S3method(print,BranchGLMCIs)
S3method(print,BranchGLMROC)
S3method(print,BranchGLMTable)
S3method(print,BranchGLMVI)
S3method(print,BranchGLMVI.boot)
S3method(print,BranchGLMVS)
S3method(print,summary.BranchGLMVS)
S3method(residuals,BranchGLM)
S3method(sigma,BranchGLM)
S3method(summary,BranchGLMVS)
S3method(vcov,BranchGLM)
export(AUC)
export(BranchGLM)
export(BranchGLM.fit)
export(Cindex)
export(MultipleROCCurves)
export(ROC)
export(Table)
export(VariableImportance)
export(VariableImportance.boot)
export(VariableSelection)
export(fit.BranchGLMVS)
export(plotCI)
import(graphics)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(methods,is)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(BranchGLM, .registration = TRUE)
